Package: SIMLR
Version: 1.39.0
Date: 2026-03-09
Title: Single-cell Interpretation via Multi-kernel LeaRning (SIMLR)
Authors@R: c(person("Daniele", "Ramazzotti", role=c("aut"),email="daniele.ramazzotti@unimib.it",
                comment = c(ORCID = "0000-0002-6087-2666")),
             person("Bo", "Wang", role=c("aut"), email="wangbo.yunze@gmail.com"),
             person("Luca", "De Sano", role=c("cre","aut"), email="luca.desano@gmail.com",
                comment = c(ORCID = "0000-0002-9618-3774")),
             person("Serafim", "Batzoglou", role=c("ctb")))
Depends: R (>= 4.1.0),
Imports: parallel, Matrix, stats, methods, Rcpp, pracma, RcppAnnoy,
        RSpectra
Suggests: BiocGenerics, BiocStyle, testthat, knitr, igraph
Description: 
    Single-cell RNA-seq technologies enable high throughput gene expression measurement of individual cells, and allow the discovery of heterogeneity within cell populations. Measurement of cell-to-cell gene expression similarity is critical for the identification, visualization and analysis of cell populations. However, single-cell data introduce challenges to conventional measures of gene expression similarity because of the high level of noise, outliers and dropouts. We develop a novel similarity-learning framework, SIMLR (Single-cell Interpretation via Multi-kernel LeaRning), which learns an appropriate distance metric from the data for dimension reduction, clustering and visualization. 
Encoding: UTF-8
License: file LICENSE
URL: https://github.com/BatzoglouLabSU/SIMLR
BugReports: https://github.com/BatzoglouLabSU/SIMLR
biocViews: ImmunoOncology, Clustering, GeneExpression, Sequencing,
        SingleCell
RoxygenNote: 7.3.3
LinkingTo: Rcpp
NeedsCompilation: yes
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/SIMLR
git_branch: devel
git_last_commit: dd6fe99
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-29 01:10:05 UTC; biocbuild
Author: Daniele Ramazzotti [aut] (ORCID:
    <https://orcid.org/0000-0002-6087-2666>),
  Bo Wang [aut],
  Luca De Sano [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-9618-3774>),
  Serafim Batzoglou [ctb]
Maintainer: Luca De Sano <luca.desano@gmail.com>
