# Generated by roxygen2: do not edit by hand

export(SIMLR)
export(SIMLR_Estimate_Number_of_Clusters)
export(SIMLR_Feature_Ranking)
export(SIMLR_Large_Scale)
import(Matrix)
import(Rcpp)
import(RcppAnnoy)
importFrom(RSpectra,eigs_sym)
importFrom(methods,is)
importFrom(methods,new)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(pracma,orth)
importFrom(stats,dnorm)
importFrom(stats,kmeans)
importFrom(stats,pbeta)
importFrom(stats,rnorm)
useDynLib(SIMLR,projsplx)
