# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.Rtsne_cpp <- function(I, J, V, no_dims_in, perplexity_in, theta_in, verbose, max_iter, Y_in, init) {
    .Call('_SIMLR_Rtsne_cpp', PACKAGE = 'SIMLR', I, J, V, no_dims_in, perplexity_in, theta_in, verbose, max_iter, Y_in, init)
}

